/*****************************************************************************
* ref18_01/initiator/iscsi_config.c
*
* This file contains the core functions to initiate session to the target.
* The file when compiled generates a user-level program which calls
* iscsi_initiator.c functions through /proc filesystem interface.
*
*   IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
*   By downloading, copying, installing or using the software you agree to this
*   license. If you do not agree to this license, do not download, install,
*   copy or use the software.
*
*   License Agreement
*
*   Copyright (C) 2001  Inter-Operability Lab.(IOL),Univ. of New Hampshire(UNH),
*                       Durham, NH 03824
*
*   Redistribution and use in source and binary forms, with or without
*   modification, are permitted provided that the following conditions are met:
*
*   -Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
*
*   -Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
*
*   The name of IOL/UNH may not be used to endorse or promote products derived
*   from this software without specific prior written permission.
*
*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
*   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
*   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL IOL,UNH OR CONTRIBUTORS
*   BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,PROCUREMENT OF
*   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
*   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*   POSSIBILITY OF SUCH DAMAGE.
*
*****************************************************************************/

/*	iscsi_config.c */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <asm/param.h>

#include "../common/debug.h"
#include "../common/iscsi_common.h"

#define DEFAULT_HOST_NO		0
#define DEFAULT_TARGET_NO	0
#define DEFAULT_LUN_NO		0
#define DEFAULT_CID_NO		0
#define WHITE_SPACE	" \t\v\f\n\r"

char	server_node[MAXHOSTNAMELEN];

/*	write a magic line to the proc file system to cause the scsi
	mid-level to redo the inquiry for our device */
void do_inquiry( int add, int host_no, int target_no, int lun_no )
	{
	int			fd, n, len;
	static char	*file_name = "/proc/scsi/scsi";
	char		text_line[128];

TRACE(TRACE_DEBUG, "Entering do_inquiry\n");

	if( (fd = open(file_name, O_RDWR)) < 0 )
		{
		perror(file_name);
		return;
		}

	len = sprintf(text_line, "scsi %s-single-device %d 0 %d %d",
											add ? "add" : "remove", host_no, target_no, lun_no);

	len++; /* for writing the last null character */

	if( (n = write(fd, text_line, len)) < 0 )
		perror(file_name);
	else if( n != len  &&  n != 0 )
		printf("%s: wrote %d bytes, expected %d\n", file_name, n, len);
	close(fd);
	
TRACE(TRACE_DEBUG, "Leaving do_inquiry\n");
	}


/*	Connects to a remote target by writing an ip address to a /proc file.
	If the ip address is 0, a disconnect will result.
	Returns	=0 if connect/disconnect worked ok for this target
			<0 if connect/disconnect failed
			>0 if just adding/removing another lun from this target
*/
int do_connect( int new_ip_address, int port_no, int host_no, 
				int target_no, int lun_no, int cid_no )
	{
	int		fd, n, len, result;
	static char	file_name[64];
	static char	text_line[128];

	TRACE(TRACE_DEBUG, "Entering do_connect\n");

	sprintf(file_name, "/proc/scsi/iscsi_initiator/%d", host_no);

	result = -1;
	if( (fd = open(file_name, O_RDWR )) < 0 )
		{
		perror(file_name);
		}
	else
		{

        len = sprintf(text_line, "iscsi_initiator ip 0x%08x port %d target %d "
						"lun %d cid %d", 
						new_ip_address, port_no,  target_no, lun_no, cid_no);

		TRACE(TRACE_DEBUG,"writing string: %s of length: %d\n",text_line,len);

		if( (n = write(fd, text_line, len)) < 0 )
			perror(file_name);
		else if( n != len && n != 0 )
			printf("%s: wrote %d bytes, expected %d\n", file_name, n, len);
		else
			result = n;
		close(fd);
		}

TRACE(TRACE_DEBUG, "Leaving do_connect, result %d\n", result);
	return result;
	}


void print_ip_address( int address, char *message )
	{
	struct in_addr	my_in_addr;

TRACE(TRACE_DEBUG, "Entering print_ip_address\n");
	my_in_addr.s_addr = htonl(address);
	printf("%13s address: %#010x -- %s\n",
									message, address, inet_ntoa(my_in_addr));

TRACE(TRACE_DEBUG, "Leaving print_ip_address\n");
	}


/* converts from a dns name to the ip address in host byte order, or 0 on err */
int cnv_from_dns_name( char *dns_name )
	{
	struct hostent	*node_ptr;

TRACE(TRACE_DEBUG, "Entering cnv_from_dns_name\n");
	/*	get structure for host node with dns name */
	if( (node_ptr = gethostbyname(dns_name)) == NULL )
		{
		herror(dns_name);
		return 0;
		}

	strcpy(server_node, node_ptr->h_name);

TRACE(TRACE_DEBUG, "Leaving cnv_from_dns_name\n");
	/*	extract the ip address of the remote host node */
	return ntohl(*(int *)node_ptr->h_addr);
	}


int cnv_from_ip_address( int ip_address )
	{
	struct hostent	*node_ptr;
	struct in_addr	my_in_addr;


TRACE(TRACE_DEBUG, "Entering cnv_from_ip_address\n");
	my_in_addr.s_addr = htonl(ip_address);

	/*	get structure for host node with ip address */
	if( (node_ptr = gethostbyaddr( (char *)&my_in_addr, 4, AF_INET)) == NULL )
		{/* this ip address is not known to the Name service.
			So just use the dotted-decimal notation as the name. */
		sprintf(server_node, "%s", inet_ntoa(my_in_addr));
		}
	else
		{/* found a DNS name for this ip address, use it. */
		strcpy(server_node, node_ptr->h_name);
		}

TRACE(TRACE_DEBUG, "Leaving cnv_from_ip_address, server_node: %s\n",
																server_node);
	return ip_address;
	}


#define	UP	0
#define	DOWN	1


int main( int argc, char *argv[] )
	{
	int				ip_address, host_no, target_no, lun_no, port_no, cid_no;
	int				updown, i;
	char			*endptr;
	/*****
	char			local_node[MAXHOSTNAMELEN];
	*****/

	if( argc < 2 )
		{
		printf("Usage: iscsi_config up/down [ip=address_or_name] [host=number] "
				"[port=number] [target=number] [lun=number] [cid=number]\n");
		exit(EXIT_FAILURE);
		}

	if( strcmp(argv[1], "up") == 0 )
		updown = UP;
	else if( strcmp(argv[1], "down") == 0 )
		updown = DOWN;
	else
		{
		printf("parameter 1 must be either \"up\" or \"down\", not \"%s\"\n",
															argv[1]);
		exit(EXIT_FAILURE);
		}

	ip_address = 0;

	/*	get the DNS name of the local host node on which we are running */
	/*****
	if( gethostname(local_node, MAXHOSTNAMELEN) < 0 )
		{
		perror("gethostname");
		exit(EXIT_FAILURE);
		}
	*****/

	/*	get ip address of remote host node on which server resides */
	/*****
	if( (ip_address = cnv_from_dns_name(local_node)) == 0 )
		exit(EXIT_FAILURE);
	*****/

	port_no = ISCSI_PORT;
	host_no = DEFAULT_HOST_NO;
	target_no = DEFAULT_TARGET_NO;
	lun_no = DEFAULT_LUN_NO;
	cid_no = DEFAULT_CID_NO;

	for( i = 2;  i < argc;  i++ )
		{
		if( strncmp(argv[i], "ip=", 3) == 0 )
			{/* dns name or ip address of remote server */
			if( isalpha(argv[i][3]) )
				{/* dns name */
				if( (ip_address=cnv_from_dns_name(&argv[i][3]))==0 )
					exit(EXIT_FAILURE);
				}
			else if( isdigit(argv[i][3]) )
				{/* numeric value */
				if( strchr(argv[i],'.') != NULL )
					{/* dotted-decimal ip address */
					if( (ip_address = ntohl(inet_addr(&argv[i][3]))) == 0  ||
						cnv_from_ip_address(ip_address) == 0 )
						{
						printf("illegal ip address \"%s\"\n", &argv[i][3]);
						exit(EXIT_FAILURE);
						}
					}
				else
					{
					errno = 0;
					ip_address = strtoul(&argv[i][3], &endptr, 0);
					if( errno!=0 || strspn(endptr,WHITE_SPACE)!=strlen(endptr)||
						cnv_from_ip_address(ip_address) == 0 )
						{
						printf("illegal number as ip address \"%s\"\n", &argv[i][3]);
						exit(EXIT_FAILURE);
						}
					}
				}
			else
				{
				printf("illegal ip address or dns name \"%s\"\n", &argv[i][3]);
				exit(EXIT_FAILURE);
				}
			}
		else if( strncmp(argv[i], "port=", 5) == 0 )
			{/* port number of target */
			errno = 0;
			port_no = strtoul(&argv[i][5], &endptr, 0);
			if( errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr) )
				{
				printf("illegal port number \"%s\"\n", &argv[i][5]);
				exit(EXIT_FAILURE);
				}
			}
		else if( strncmp(argv[i], "host=", 5) == 0 )
			{/* number of host adaptor */
			errno = 0;
			host_no = strtoul(&argv[i][5], &endptr, 0);
			if( errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr) )
				{
				printf("illegal host number \"%s\"\n", &argv[i][5]);
				exit(EXIT_FAILURE);
				}
			}
		else if( strncmp(argv[i], "target=", 7) == 0 )
			{/* id number of target */
			errno = 0;
			target_no = strtoul(&argv[i][7], &endptr, 0);
			if( errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr) )
				{
				printf("illegal target number \"%s\"\n", &argv[i][7]);
				exit(EXIT_FAILURE);
				}
			}
		else if( strncmp(argv[i], "lun=", 4) == 0 )
			{/* number of lun */
			errno = 0;
			lun_no = strtoul(&argv[i][4], &endptr, 0);
			if( errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr) )
				{
				printf("illegal lun number \"%s\"\n", &argv[i][4]);
				exit(EXIT_FAILURE);
				}
			}
		else if( strncmp(argv[i], "cid=", 4) == 0 )
			{/* number of cid */
			errno = 0;
			cid_no = strtoul(&argv[i][4], &endptr, 0);
			if( errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr) )
				{
				printf("illegal cid number \"%s\"\n", &argv[i][4]);
				exit(EXIT_FAILURE);
				}
			}
		else
			{
			printf("illegal parameter \"%s\"\n", argv[i]);
			exit(EXIT_FAILURE);
			}
		}

	if( updown == UP )
		{
		if( do_connect(ip_address,port_no,host_no,target_no,lun_no,cid_no) > 0 )
			{
			do_inquiry(1, host_no, target_no, lun_no);
			}
		}
	if( updown == DOWN )
		{
		if( do_connect(0, port_no, host_no, target_no, lun_no, cid_no) > 0 )
				{
				do_inquiry(0, host_no, target_no, lun_no);
				}
		}

	exit(EXIT_SUCCESS);
	}
